#include <apop.h>
apop_model econ_101_analytic;
apop_model econ_101;

#define fget(r, c) apop_data_get(fixed_params, (r), (c))

static apop_model * econ101_analytic_est(apop_data * fixed_params, apop_model *pin){
  apop_model   *est    = apop_model_copy(econ_101_analytic);
  double      budget      = fget(0, -1), p1  = fget(0, 0), p2   = fget(1, 0),
            alpha       = fget(0, 1), beta = fget(1, 1);
  double  x2  = budget/(alpha/beta + 1)/p2,
          x1  = (budget - p2*x2)/p1;
    est->data   = fixed_params;
    est->parameters = apop_data_alloc(2,0,0);
    apop_data_fill(est->parameters, x1, x2);
    est->llikelihood = log(econ_101.p(fixed_params, est));
    return est;
}

static void econ101_analytic_score(apop_data *fixed_params, gsl_vector *gradient, apop_model *m){
  double      x1          = apop_data_get(m->parameters, 0, -1);
  double      x2          = apop_data_get(m->parameters, 1, -1);
  double      alpha       = fget(0, 1), beta = fget(1, 1);
    gsl_vector_set(gradient, 0, alpha*pow(x1,alpha-1)*pow(x2,beta));
    gsl_vector_set(gradient, 1, beta*pow(x2,beta-1)*pow(x1,alpha));
}

apop_model econ_101_analytic = {"Analytically solve Cobb-Douglass maximization subject to a budget constraint", 
    .vbase =2, .estimate=econ101_analytic_est, .score = econ101_analytic_score};
